@@ -1,5 +1,6 @@
 """Module to analyze transitions during free recall."""
 
+import itertools
 import numpy as np
 from scipy import stats
 import pandas as pd
@@ -98,6 +99,9 @@ def transitions_masker(
 
     Yields
     ------
+    output : int
+        Output position of this transition. The first transition is 1.
+
     prev : object
         Output value for the "from" item on this transition.
 
@@ -109,16 +113,17 @@ def transitions_masker(
 
     Examples
     --------
+    >>> from psifr import transitions
     >>> pool = [1, 2, 3, 4, 5, 6]
     >>> recs = [6, 2, 3, 6, 1, 4]
-    >>> masker = transitions_masker(
+    >>> masker = transitions.transitions_masker(
     ...     pool_items=pool, recall_items=recs, pool_output=pool, recall_output=recs
     ... )
-    >>> for prev, curr, poss in masker:
-    ...     print(prev, curr, poss)
-    6 2 [1 2 3 4 5]
-    2 3 [1 3 4 5]
-    1 4 [4 5]
+    >>> for output, prev, curr, poss in masker:
+    ...     print(output, prev, curr, poss)
+    1 6 2 [1 2 3 4 5]
+    2 2 3 [1 3 4 5]
+    5 1 4 [4 5]
     """
     n = 0
     pool_items = pool_items.copy()
@@ -159,7 +164,131 @@ def transitions_masker(
                 ind = np.repeat(ind, poss.shape)
             poss = poss[ind]
         n += 1
-        yield prev, curr, poss
+        yield n, prev, curr, poss
+
+
+def sequences_masker(
+    n_transitions,
+    pool_items,
+    recall_items,
+    pool_output,
+    recall_output,
+    pool_test=None,
+    recall_test=None,
+    test=None,
+):
+    """
+    Yield sequences of adjacent included transitions.
+
+    Parameters
+    ----------
+    n_transitions : int
+        Number of transitions to include in yielded sequences.
+
+    pool_items : list
+        Items available for recall. Order does not matter. May contain
+        repeated values. Item identifiers must be unique within pool.
+
+    recall_items : list
+        Recalled items in output position order.
+
+    pool_output : list
+        Output values for pool items. Must be the same order as pool.
+
+    recall_output : list
+        Output values in output position order.
+
+    pool_test : list, optional
+        Test values for items available for recall. Must be the same
+        order as pool.
+
+    recall_test : list, optional
+        Test values for items in output position order.
+
+    test : callable, optional
+        Used to test whether individual transitions should be included,
+        based on test values.
+
+            test(prev, curr) - test for included transition
+
+            test(prev, poss) - test for included possible transition
+
+    Yields
+    ------
+    output : int
+        Output positions of included transitions. The first transition
+        is 1.
+
+    prev : list
+        Output values for the "from" item in included transitions.
+
+    curr : list
+        Output values for the "to" item in included transitions.
+
+    poss : list of numpy.ndarray
+        Output values for all possible valid "to" items in included
+        transitions.
+
+    See Also
+    --------
+    transitions_masker : Yield included transitions.
+
+    Examples
+    --------
+    >>> from psifr import transitions
+    >>> pool = [1, 2, 3, 4, 5, 6]
+    >>> recs = [6, 2, 3, 6, 1, 4, 5]
+    >>> masker = transitions.sequences_masker(
+    ...     2, pool_items=pool, recall_items=recs, pool_output=pool, recall_output=recs
+    ... )
+    >>> for output, prev, curr, poss in masker:
+    ...     print(output, prev, curr, poss)
+    [1, 2] [6, 2] [2, 3] [array([1, 2, 3, 4, 5]), array([1, 3, 4, 5])]
+    [5, 6] [1, 4] [4, 5] [array([4, 5]), array([5])]
+
+    >>> pool = [1, 2, 3, 4]
+    >>> recs = [4, 3, 1, 2]
+    >>> masker = transitions.sequences_masker(
+    ...     3, pool_items=pool, recall_items=recs, pool_output=pool, recall_output=recs
+    ... )
+    >>> for output, prev, curr, poss in masker:
+    ...     print(output, prev, curr, poss)
+    [1, 2, 3] [4, 3, 1] [3, 1, 2] [array([1, 2, 3]), array([1, 2]), array([2])]
+    """
+    masker = transitions_masker(
+        pool_items,
+        recall_items,
+        pool_output,
+        recall_output,
+        pool_test=pool_test,
+        recall_test=recall_test,
+        test=test,
+    )
+    s_output = []
+    s_prev = []
+    s_curr = []
+    s_poss = []
+    prev_output = 0
+    sequence_len = 0
+    for output, prev, curr, poss in masker:
+        if (output - prev_output) == 1:
+            s_output.append(output)
+            s_prev.append(prev)
+            s_curr.append(curr)
+            s_poss.append(poss)
+            sequence_len += 1
+        else:
+            # a break in the chain
+            s_output = [output]
+            s_prev = [prev]
+            s_curr = [curr]
+            s_poss = [poss]
+            sequence_len = 1
+        prev_output = output
+
+        if sequence_len >= n_transitions:
+            ind = slice(sequence_len - n_transitions, None)
+            yield s_output[ind], s_prev[ind], s_curr[ind], s_poss[ind]
 
 
 def count_lags(
@@ -235,6 +364,7 @@ def count_lags(
     >>> recall_items = [[4, 2, 3, 1]]
     >>> actual, possible = transitions.count_lags(4, pool_items, recall_items)
     >>> actual
+    lag
     -3    0
     -2    2
     -1    0
@@ -244,6 +374,7 @@ def count_lags(
      3    0
     dtype: int64
     >>> possible
+    lag
     -3    1
     -2    2
     -1    2
@@ -275,7 +406,7 @@ def count_lags(
             test,
         )
 
-        for prev, curr, poss in masker:
+        for output, prev, curr, poss in masker:
             # for this step, calculate actual lag and all possible lags
             list_actual.append(curr - prev)
             tran_poss = poss - prev
@@ -286,8 +417,169 @@ def count_lags(
     # count the actual and possible transitions for each lag
     max_lag = list_length - 1
     lags = np.arange(-max_lag, max_lag + 2)
-    actual = pd.Series(np.histogram(list_actual, lags)[0], index=lags[:-1])
-    possible = pd.Series(np.histogram(list_possible, lags)[0], index=lags[:-1])
+    index = pd.Index(lags[:-1], name='lag')
+    actual = pd.Series(np.histogram(list_actual, lags)[0], index=index)
+    possible = pd.Series(np.histogram(list_possible, lags)[0], index=index)
+    return actual, possible
+
+
+def count_lags_compound(
+    list_length,
+    pool_items,
+    recall_items,
+    pool_label=None,
+    recall_label=None,
+    pool_test=None,
+    recall_test=None,
+    test=None,
+    count_unique=False,
+):
+    """
+    Count lags conditional on the lag of the previous transition.
+
+    Parameters
+    ----------
+    list_length : int
+        Number of items in each list.
+
+    pool_items : list
+        List of the serial positions available for recall in each list.
+        Must match the serial position codes used in `recall_items`.
+
+    recall_items : list
+        List indicating the serial position of each recall in output
+        order (NaN for intrusions).
+
+    pool_label : list, optional
+        List of the positions to use for calculating lag. Default is to
+        use `pool_items`.
+
+    recall_label : list, optional
+        List of position labels in recall order. Default is to use
+        `recall_items`.
+
+    pool_test : list, optional
+         List of some test value for each item in the pool.
+
+    recall_test : list, optional
+        List of some test value for each recall attempt by output
+        position.
+
+    test : callable
+        Callable that evaluates each transition between items n and
+        n+1. Must take test values for items n and n+1 and return True
+        if a given transition should be included.
+
+    count_unique : bool, optional
+        If true, only unique values will be counted toward the possible
+        transitions. If multiple items are avilable for recall for a
+        given transition and a given bin, that bin will only be
+        incremented once. If false, all possible transitions will add
+        to the count.
+
+    Returns
+    -------
+    actual : pandas.Series
+        Count of actual lags that occurred in the recall sequence.
+
+    possible : pandas.Series
+        Count of possible lags.
+
+    See Also
+    --------
+    count_lags : Count of individual transitions.
+
+    Examples
+    --------
+    >>> from psifr import transitions
+    >>> pool_items = [[1, 2, 3]]
+    >>> recall_items = [[3, 1, 2]]
+    >>> actual, possible = transitions.count_lags_compound(3, pool_items, recall_items)
+    >>> (actual == possible).all()
+    True
+    >>> actual
+    previous  current
+    -2        -2         0
+              -1         0
+               0         0
+               1         1
+               2         0
+    -1        -2         0
+              -1         0
+               0         0
+               1         0
+               2         0
+     0        -2         0
+              -1         0
+               0         0
+               1         0
+               2         0
+     1        -2         0
+              -1         0
+               0         0
+               1         0
+               2         0
+     2        -2         0
+              -1         0
+               0         0
+               1         0
+               2         0
+    dtype: int64
+    """
+    if pool_label is None:
+        pool_label = pool_items
+
+    if recall_label is None:
+        recall_label = recall_items
+
+    prev_actual = []
+    prev_possible = []
+    curr_actual = []
+    curr_possible = []
+    for i, recall_items_list in enumerate(recall_items):
+        # set up masker to filter pairs of transitions
+        pool_test_list = None if pool_test is None else pool_test[i]
+        recall_test_list = None if recall_test is None else recall_test[i]
+        masker = sequences_masker(
+            2,
+            pool_items[i],
+            recall_items_list,
+            pool_label[i],
+            recall_label[i],
+            pool_test_list,
+            recall_test_list,
+            test,
+        )
+
+        for output, prev, curr, poss in masker:
+            # calculate lag of prior transition
+            prev_lag = curr[-2] - prev[-2]
+
+            # actual and possible lags of current transition
+            curr_lag = curr[-1] - prev[-1]
+            poss_lag = poss[-1] - prev[-1]
+
+            # add lags to count
+            prev_actual.append(prev_lag)
+            curr_actual.append(curr_lag)
+            if count_unique:
+                poss_lag = np.unique(poss_lag)
+            prev_possible.extend([prev_lag] * len(poss_lag))
+            curr_possible.extend(poss_lag)
+
+    # define possible combinations of previous and current lags
+    max_lag = list_length - 1
+    bins = np.arange(-max_lag, max_lag + 2)
+    lags = bins[:-1]
+    compound_lags = list(itertools.product(lags, lags))
+    index = pd.MultiIndex.from_tuples(compound_lags, names=['previous', 'current'])
+
+    # count the actual and possible transitions for each (lag, lag)
+    # combination
+    count_actual = np.histogram2d(prev_actual, curr_actual, bins)[0].astype(int)
+    actual = pd.Series(count_actual.ravel(), index=index)
+    count_possible = np.histogram2d(prev_possible, curr_possible, bins)[0].astype(int)
+    possible = pd.Series(count_possible.ravel(), index=index)
     return actual, possible
 
 
@@ -374,7 +666,7 @@ def rank_lags(
             test,
         )
 
-        for prev, curr, poss in masker:
+        for output, prev, curr, poss in masker:
             actual = np.abs(curr - prev)
             possible = np.abs(poss - prev)
             rank.append(1 - percentile_rank(actual, possible))
@@ -447,6 +739,7 @@ def count_distance(
 
     Examples
     --------
+    >>> import numpy as np
     >>> from psifr import transitions
     >>> distances = np.array([[0, 1, 2, 2], [1, 0, 2, 2], [2, 2, 0, 3], [2, 2, 3, 0]])
     >>> edges = np.array([0.5, 1.5, 2.5, 3.5])
@@ -483,7 +776,7 @@ def count_distance(
             recall_test_list,
             test,
         )
-        for prev, curr, poss in masker:
+        for output, prev, curr, poss in masker:
             prev = int(prev)
             curr = int(curr)
             poss = poss.astype(int)
@@ -558,9 +851,9 @@ def rank_distance(
 
     Examples
     --------
+    >>> import numpy as np
     >>> from psifr import transitions
     >>> distances = np.array([[0, 1, 2, 2], [1, 0, 2, 2], [2, 2, 0, 3], [2, 2, 3, 0]])
-    >>> edges = np.array([0.5, 1.5, 2.5, 3.5])
     >>> pool_items = [[1, 2, 3, 4]]
     >>> recall_items = [[4, 2, 3, 1]]
     >>> pool_index = [[0, 1, 2, 3]]
@@ -583,7 +876,7 @@ def rank_distance(
             recall_test_list,
             test,
         )
-        for prev, curr, poss in masker:
+        for output, prev, curr, poss in masker:
             prev = int(prev)
             curr = int(curr)
             poss = poss.astype(int)
@@ -593,6 +886,115 @@ def rank_distance(
     return rank
 
 
+def rank_distance_shifted(
+    distances,
+    max_shift,
+    pool_items,
+    recall_items,
+    pool_index,
+    recall_index,
+    pool_test=None,
+    recall_test=None,
+    test=None,
+):
+    """
+    Calculate percentile rank of shifted distances.
+
+    Parameters
+    ----------
+    distances : numpy.array
+        Items x items matrix of pairwise distances or similarities.
+
+    max_shift : int
+        Maximum number of items back for which to rank distances.
+
+    pool_items : list of list
+        Unique item codes for each item in the pool available for recall.
+
+    recall_items : list of list
+        Unique item codes of recalled items.
+
+    pool_index : list of list
+        Index of each item in the distances matrix.
+
+    recall_index : list of list
+        Index of each recalled item.
+
+    pool_test : list of list, optional
+        Test value for each item in the pool.
+
+    recall_test : list of list, optional
+        Test value for each recalled item.
+
+    test : callable
+        Called as test(prev, curr) or test(prev, poss) to screen
+        actual and possible transitions, respectively.
+
+    Returns
+    -------
+    rank : numpy.ndarray
+        [transitions x max_shift] array with distance percentile ranks.
+        The rank is 0 if the distance was the largest of the available
+        transitions, and 1 if the distance was the smallest. Ties are
+        assigned to the average percentile rank.
+
+    See Also
+    --------
+    rank_distance - Percentile rank of transition distances relative
+        to the immediately preceding item only.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from psifr import transitions
+    >>> distances = np.array(
+    ...     [
+    ...         [0, 1, 2, 2, 2],
+    ...         [1, 0, 2, 2, 2],
+    ...         [2, 2, 0, 3, 3],
+    ...         [2, 2, 3, 0, 2],
+    ...         [2, 2, 3, 2, 0],
+    ...     ]
+    ... )
+    >>> pool_items = [[1, 2, 3, 4, 5]]
+    >>> recall_items = [[4, 2, 3, 1]]
+    >>> pool_index = [[0, 1, 2, 3, 4]]
+    >>> recall_index = [[3, 1, 2, 0]]
+    >>> transitions.rank_distance_shifted(
+    ...     distances, 2, pool_items, recall_items, pool_index, recall_index
+    ... )
+    array([[0.  , 0.25],
+           [1.  , 1.  ]])
+    """
+    rank = []
+    for i in range(len(recall_items)):
+        pool_test_list = None if pool_test is None else pool_test[i]
+        recall_test_list = None if recall_test is None else recall_test[i]
+        masker = sequences_masker(
+            max_shift,
+            pool_items[i],
+            recall_items[i],
+            pool_index[i],
+            recall_index[i],
+            pool_test_list,
+            recall_test_list,
+            test,
+        )
+        for s_output, s_prev, s_curr, s_poss in masker:
+            curr = int(s_curr[-1])
+            poss = s_poss[-1].astype(int)
+            rank_shift = []
+            for j in range(max_shift, 0, -1):
+                # rank previous to current distance for this shift
+                prev = int(s_prev[-j])
+                actual = distances[prev, curr]
+                possible = distances[prev, poss]
+                rank_shift.append(1 - percentile_rank(actual, possible))
+            rank.append(rank_shift)
+    rank = np.array(rank)
+    return rank
+
+
 def count_category(
     pool_items,
     recall_items,
@@ -669,7 +1071,7 @@ def count_category(
             test,
         )
 
-        for prev, curr, poss in masker:
+        for output, prev, curr, poss in masker:
             if prev == curr:
                 actual += 1
             if np.any(prev == poss):
@@ -697,7 +1099,7 @@ def count_pairs(
             test,
         )
 
-        for prev, curr, poss in masker:
+        for output, prev, curr, poss in masker:
             actual[prev, curr] += 1
             possible[prev, poss] += 1
     return actual, possible
